@@ -1,8 +1,8 @@
-## DNA.R (2017-08-28)
+## DNA.R (2018-09-22)
 
 ##   Manipulations and Comparisons of DNA and AA Sequences
 
-## Copyright 2002-2017 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
+## Copyright 2002-2018 Emmanuel Paradis, 2015 Klaus Schliep, 2017 Franz Krah
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -124,6 +124,7 @@ as.list.DNAbin <- function(x, ...)
     if (is.list(x)) return(x)
     if (is.null(dim(x))) obj <- list(x) # cause is.vector() doesn't work
     else { # matrix
+        class(x) <- NULL
         n <- nrow(x)
         obj <- vector("list", n)
         for (i in seq_len(n)) obj[[i]] <- x[i, , drop = TRUE]
@@ -153,7 +154,6 @@ rbind.DNAbin <- function(...)
 cbind.DNAbin <-
     function(..., check.names = TRUE, fill.with.gaps = FALSE,
              quiet = FALSE)
-### works only with matrices for the moment
 {
     obj <- list(...)
     n <- length(obj)
@@ -164,7 +164,11 @@ cbind.DNAbin <-
     NR <- unlist(lapply(obj, nrow))
     for (i in 1:n) class(obj[[i]]) <- NULL
     if (check.names) {
-        nms <- unlist(lapply(obj, rownames))
+        NMS <- lapply(obj, rownames)
+        for (i in 1:n)
+            if (anyDuplicated(NMS[[i]]))
+                stop("Duplicated rownames in matrix ", i, ": see ?cbind.DNAbin")
+        nms <- unlist(NMS)
         if (fill.with.gaps) {
             NC <- unlist(lapply(obj, ncol))
             nms <- unique(nms)
@@ -173,9 +177,8 @@ cbind.DNAbin <-
             from <- 1
             for (i in 1:n) {
                 to <- from + NC[i] - 1
-                tmp <- rownames(obj[[i]])
-                nmsi <- tmp[tmp %in% nms]
-                ans[nmsi, from:to] <- obj[[i]][nmsi, , drop = FALSE]
+                k <- match(NMS[[i]], nms)
+                ans[k, from:to] <- obj[[i]]
                 from <- to + 1
             }
         } else {
@@ -260,7 +263,14 @@ print.DNAbin <- function(x, printlen = 6, digits = 3, ...)
     if (nTot <= 1e7) {
         cat("Base composition:\n")
         print(round(base.freq(x), digits))
-    } else cat("More than 10 million nucleotides: not printing base composition\n")
+    } else {
+        cat("More than 10 million bases: not printing base composition\n")
+    }
+    if (nTot > 1) {
+        k <- floor(log(nTot, 1000))
+        units <- c("bases", "kb", "Mb", "Gb", "Tb", "Pb", "Eb")
+        cat("(Total: ", round(nTot/1000^k, 2), " ", units[k + 1], ")\n", sep = "")
+    }
 }
 
 as.DNAbin <- function(x, ...) UseMethod("as.DNAbin")
@@ -495,7 +505,7 @@ image.DNAbin <-
         brks <- c(-0.5, brks)
     }
     yaxt <- if (show.labels) "n" else "s"
-    image.default(1:s, 1:n, t(y[n:1, ]), col = co, xlab = xlab,
+    image.default(1:s, 1:n, t(y[n:1, , drop = FALSE]), col = co, xlab = xlab,
                   ylab = ylab, yaxt = yaxt, breaks = brks, ...)
     if (show.labels)
         mtext(rownames(x), side = 2, line = 0.1, at = n:1,
@@ -566,26 +576,50 @@ where <- function(x, pattern)
 
 ## conversions from BioConductor:
 
+## DNA:
 .DNAString2DNAbin <- function(from)
-    as.DNAbin.character(strsplit(tolower(as.character(from)), "")[[1]])
+    .Call("charVectorToDNAbinVector", as.character(from))
 
-.DNAStringSet2DNAbin <- function(from)
-    structure(lapply(from, .DNAString2DNAbin), class = "DNAbin")
+as.DNAbin.DNAString <- function(x, ...)
+{
+    res <- list(.DNAString2DNAbin(x))
+    class(res) <- "DNAbin"
+    res
+}
 
-.DNAAlignment2DNAbinMatrix <- function(from)
-    as.matrix(.DNAStringSet2DNAbin(as(from, "DNAStringSet")))
+as.DNAbin.DNAStringSet <- function(x, ...)
+{
+    res <- lapply(x, .DNAString2DNAbin)
+    class(res) <- "DNAbin"
+    res
+}
 
-## S3 versions:
-as.DNAbin.DNAString <- function(x, ...) .DNAString2DNAbin(x)
-as.DNAbin.DNAStringSet <- function(x, ...) .DNAStringSet2DNAbin(x)
-as.DNAbin.PairwiseAlignmentsSingleSubject <- function(x, ...) .DNAAlignment2DNAbinMatrix(x)
-as.DNAbin.DNAMultipleAlignment <- function(x, ...) .DNAAlignment2DNAbinMatrix(x)
+as.DNAbin.DNAMultipleAlignment <- function(x, ...)
+    as.matrix(as.DNAbin.DNAStringSet(as(x, "DNAStringSet")))
 
-## S4 versions:
-## setAs("DNAString", "DNAbin", .DNAString2DNAbin)
-## setAs("DNAStringSet", "DNAbin", .DNAStringSet2DNAbin)
-## setAs("PairwiseAlignmentsSingleSubject", "DNAbin", .DNAAlignment2DNAbinMatrix)
-## setAs("DNAMultipleAlignment", "DNAbin", .DNAAlignment2DNAbinMatrix)
+as.DNAbin.PairwiseAlignmentsSingleSubject <- function(x, ...)
+    as.DNAbin.DNAMultipleAlignment(x)
+
+## AA:
+.AAString2AAbin <- function(from)
+    charToRaw(as.character(from))
+
+as.AAbin.AAString <- function(x, ...)
+{
+    res <- list(.AAString2AAbin(x))
+    class(res) <- "AAbin"
+    res
+}
+
+as.AAbin.AAStringSet <- function(x, ...)
+{
+    res <- lapply(x, .AAString2AAbin)
+    class(res) <- "AAbin"
+    res
+}
+
+as.AAbin.AAMultipleAlignment <- function(x, ...)
+    as.matrix(as.AAbin.AAStringSet(as(x, "AAStringSet")))
 
 complement <- function(x)
 {
@@ -699,8 +733,11 @@ as.AAbin.character <- function(x, ...)
 
 labels.AAbin <- function(object, ...) labels.DNAbin(object, ...)
 
+## TO BE MOVED TO phangorn LATER
 if (getRversion() >= "2.15.1") utils::globalVariables("phyDat")
 as.phyDat.AAbin <- function(x, ...) phyDat(as.character(x), type = "AA")
+## \alias{as.phyDat.AAbin}
+## \method{as.phyDat}{AAbin}(x, \dots)
 
 dist.aa <- function(x, pairwise.deletion = FALSE, scaled = FALSE)
 {
@@ -1079,3 +1116,24 @@ as.matrix.AAbin <- function(x, ...)
   class(y) <- "AAbin"
   y
 }
+
+rDNAbin <- function(n, nrow, ncol, base.freq = rep(0.25, 4), prefix = "Ind_")
+{
+    foo <- function(n, prob) {
+        vec <- as.raw(._bs_[1:4])
+        vec[sample.int(4L, n, TRUE, prob, FALSE)]
+    }
+    base.freq <- if (all(base.freq == 0.25)) NULL else base.freq[c(1, 3, 2, 4)]
+    if (missing(n)) {
+        if (missing(nrow) && missing(ncol))
+            stop("nrow and ncol should be given if n is missing")
+        res <- foo(nrow * ncol, base.freq)
+        dim(res) <- c(nrow, ncol)
+        rownames(res) <- paste0(prefix, 1:nrow)
+    } else {
+        res <- lapply(n, foo, prob = base.freq)
+        names(res) <- paste0(prefix, seq_along(n))
+    }
+    class(res) <- "DNAbin"
+    res
+}
